from __future__ import annotations

from typing import Literal, Mapping, NewType

from typing_extensions import NotRequired, Unpack

from haitch._attrs import AttributeValue, GlobalAttrs
from haitch._void_elements._void_element import VoidElement

ImgElement = NewType("ImgElement", VoidElement)
"""An `<img>` element."""


def img(
    *,
    extra_attrs: Mapping[str, AttributeValue] = {},
    **attrs: Unpack[ImgAttrs],
) -> ImgElement:
    """Embeds an image into the document.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img>
    """
    el = VoidElement("img")(**attrs, **extra_attrs)
    return ImgElement(el)


class ImgAttrs(GlobalAttrs):
    """Attributes for the `<img>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img>
    """

    src: str
    """Contains the path to the image you want to embed.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#src>
    """

    alt: str
    """Holds a textual replacement for the image.

    Note: Browsers do not always display images. There are a number of situations in
    which a browser might not display images, such as:

        - Non-visual browsers (such as those used by people with visual impairments)
        - The user chooses not to display images (saving bandwidth, privacy reasons)
        - The image is invalid or an unsupported type

    In these cases, the browser may replace the image with the text in the element's
    alt attribute. For these reasons and others, provide a useful value for alt
    whenever possible.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#alt>
    """

    crossorigin: NotRequired[Literal["anonymous", "use-credentials"]]
    """Indicates if the fetching of the image must be done using a CORS request.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#crossorigin>
    """

    decoding: NotRequired[Literal["sync", "async", "auto"]]
    """Provides a hint to the browser as to whether it should perform image decoding.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#decoding>
    """

    elementtiming: NotRequired[str]
    """Marks the image for observation by the PerformanceElementTiming API.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#elementtiming>
    """

    fetchpriority: NotRequired[Literal["high", "low", "auto"]]
    """Provides a hint of the relative priority to use when fetching the image.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#fetchpriority>
    """

    height: NotRequired[int]
    """The intrinsic height of the image in pixels.

    Note: Including height and width enables the aspect ratio of the image to be
    calculated by the browser prior to the image being loaded. This aspect ratio is used
    to reserve the space needed to display the image, reducing or even preventing a
    layout shift when the image is downloaded and painted to the screen. Reducing
    layout shift is a major component of good user experience and web performance.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#height>
    """

    ismap: NotRequired[bool]
    """Indicates that the image is part of a server-side map.

    Note: This attribute is allowed only if the `<img>` element is a descendant of
    an `<a>` element with a valid href attribute. This gives users without pointing
    devices a fallback destination.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#ismap>
    """

    loading: NotRequired[Literal["eager", "lazy"]]
    """Indicates how the browser should load the image.

    Note: Loading is only deferred when JavaScript is enabled. This is an anti-tracking
    measure, because if a user agent supported lazy loading when scripting is disabled,
    it would still be possible for a site to track a user's approximate scroll position
    throughout a session, by strategically placing images in a page's markup such that
    a server can track how many images are requested and when.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#loading>
    """

    referrerpolicy: NotRequired[
        Literal[
            "no-referrer",
            "no-referrer-when-downgrade",
            "origin",
            "origin-when-cross-origin",
            "same-origin",
            "strict-origin",
            "strict-origin-when-cross-origin",
            "unsafe-url",
        ]
    ]
    """A string indicating which referrer to use when fetching the resource.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#referrerpolicy>
    """

    sizes: NotRequired[str]
    """One or more strings separated by commas, indicating a set of source sizes.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#sizes>
    """

    srcset: NotRequired[str]
    """One or more strings separated by commas, indicating possible image sources for
    the user agent to use.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#srcset>
    """

    width: NotRequired[int]
    """The intrinsic width of the image in pixels.

    Note: Including height and width enables the aspect ratio of the image to be
    calculated by the browser prior to the image being loaded. This aspect ratio is
    used to reserve the space needed to display the image, reducing or even preventing
    a layout shift when the image is downloaded and painted to the screen. Reducing
    layout shift is a major component of good user experience and web performance.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#width>
    """

    usemap: NotRequired[str]
    """The partial URL (starting with #) of an image map associated with the element.

    Note: You cannot use this attribute if the `<img>` element is inside an `<a>` or
    `<button>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img#usemap>
    """



from __future__ import annotations

from typing import Mapping, NewType

from typing_extensions import Unpack

from haitch._attrs import AttributeValue, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

SpanElement = NewType("SpanElement", Element)
"""A `<span>` element."""


def span(
    *children: Child,
    extra_attrs: Mapping[str, AttributeValue] = {},
    **attrs: Unpack[GlobalAttrs],
) -> SpanElement:
    """Generic inline container for phrasing content.

    Span does not inherently represent anything. It can be used to group elements for
    styling purposes (using the `class` or `id` attributes), or because they share
    attribute values, such as `lang`. It should be used only when no other semantic
    element is appropriate. `<span>` is very much like a `<div>` element, but `<div>`
    is a block-level element whereas a `<span>` is an inline-level element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/span>
    """
    el = Element("span")(**attrs, **extra_attrs)(*children)
    return SpanElement(el)



import haitch as H


def test_area_element():
    el = H.area(
        shape="poly",
        coords="129,0,260,95,129,138",
        href="https://developer.mozilla.org/docs/",
    )

    got = str(el)
    want = '<area shape="poly" coords="129,0,260,95,129,138" href="https://developer.mozilla.org/docs/"/>'

    assert got == want


def test_base_element():
    got = str(H.base(href="https://example.com"))
    want = '<base href="https://example.com"/>'
    assert got == want


def test_br_element():
    got = str(H.br())
    want = "<br/>"
    assert got == want


def test_col_element():
    got = str(H.col(span=2))
    want = '<col span="2"/>'
    assert got == want


def test_embed_element():
    el = H.embed(
        type_="video/quicktime",
        src="movie.mov",
        width=640,
        height=480,
        title="Title of my video",
    )

    got = str(el)
    want = '<embed type="video/quicktime" src="movie.mov" width="640" height="480" title="Title of my video"/>'

    assert got == want


def test_hr_element():
    got = str(H.hr())
    want = "<hr/>"
    assert got == want


def test_img_element():
    got = str(H.img(src="photo.png", alt="My image"))
    want = '<img src="photo.png" alt="My image"/>'
    assert got == want


def test_input_element():
    got = str(H.input(name="my-button", type_="button"))
    want = '<input name="my-button" type="button"/>'
    assert got == want


def test_link_element():
    got = str(H.link(href="default.css", rel="stylesheet"))
    want = '<link href="default.css" rel="stylesheet"/>'
    assert got == want


def test_meta_element():
    got = str(H.meta(charset="utf-8"))
    want = '<meta charset="utf-8"/>'
    assert got == want


def test_source_element():
    got = str(H.source(src="foo.mov", type_="video/quicktime"))
    want = '<source src="foo.mov" type="video/quicktime"/>'
    assert got == want


def test_track_element():
    got = str(H.track(src="captions.vtt", kind="captions"))
    want = '<track src="captions.vtt" kind="captions"/>'
    assert got == want


def test_wbr_element():
    got = str(H.wbr())
    want = "<wbr/>"
    assert got == want


def test_void_element_with_extra_attrs():
    el = H.img(
        src="photo.png",
        alt="My image",
        extra_attrs={"foo": "bar"},
    )

    got = str(el)
    want = '<img src="photo.png" alt="My image" foo="bar"/>'

    assert got == want



from __future__ import annotations

from typing import Mapping, NewType

from typing_extensions import Unpack

from haitch._attrs import AttributeValue, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

DivElement = NewType("DivElement", Element)
"""A `<div>` element."""


def div(
    *children: Child,
    extra_attrs: Mapping[str, AttributeValue] = {},
    **attrs: Unpack[GlobalAttrs],
) -> DivElement:
    """Generic container for flow content.

    As a "pure" container, the `<div>` element does not inherently represent anything.
    Instead, it's used to group content so it can be easily styled using the `class` or
    `id` attributes, marking a section of a document as being written in a different
    language (using the `lang` attribute), and so on.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/div>
    """
    el = Element("div")(**attrs, **extra_attrs)(*children)
    return DivElement(el)



import haitch as H


def test_article_element():
    got = str(H.article("Really cool site"))
    want = "<article>Really cool site</article>"
    assert got == want


def test_aside_element():
    got = str(H.aside("Really cool aside"))
    want = "<aside>Really cool aside</aside>"
    assert got == want


def test_details_element():
    got = str(H.details(open=True)("Really cool details"))
    want = "<details open>Really cool details</details>"
    assert got == want


def test_figcaption_element():
    got = str(H.figcaption("Really cool figcaption"))
    want = "<figcaption>Really cool figcaption</figcaption>"
    assert got == want


def test_figure_element():
    got = str(H.figure("Really cool figure"))
    want = "<figure>Really cool figure</figure>"
    assert got == want


def test_footer_element():
    got = str(H.footer("Really cool footer"))
    want = "<footer>Really cool footer</footer>"
    assert got == want


def test_header_element():
    got = str(H.header("Really cool header"))
    want = "<header>Really cool header</header>"
    assert got == want


def test_main_element():
    got = str(H.main("Really cool main"))
    want = "<main>Really cool main</main>"
    assert got == want


def test_mark_element():
    got = str(H.mark("Really cool mark"))
    want = "<mark>Really cool mark</mark>"
    assert got == want


def test_nav_element():
    got = str(H.nav("Really cool nav"))
    want = "<nav>Really cool nav</nav>"
    assert got == want


def test_section_element():
    got = str(H.section("Really cool section"))
    want = "<section>Really cool section</section>"
    assert got == want


def test_summary_element():
    got = str(H.summary("Really cool summary"))
    want = "<summary>Really cool summary</summary>"
    assert got == want


def test_time_element():
    got = str(H.time(datetime="2024-06-19")("June 19th, 2024"))
    want = '<time datetime="2024-06-19">June 19th, 2024</time>'
    assert got == want



from __future__ import annotations

from typing import Mapping, NewType

from typing_extensions import Unpack

from haitch._attrs import AttributeValue, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

HeaderElement = NewType("HeaderElement", Element)
"""A `<header>` element."""


def header(
    *children: Child,
    extra_attrs: Mapping[str, AttributeValue] = {},
    **attrs: Unpack[GlobalAttrs],
) -> HeaderElement:
    """Represents introductory content, typically a group of introductory or nav aids.

    It may contain some heading elements but also a logo, a search form, an author name,
    and other elements.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/header>
    """
    el = Element("header")(**attrs, **extra_attrs)(*children)
    return HeaderElement(el)



from __future__ import annotations

from typing import Mapping, NewType

from typing_extensions import NotRequired, Unpack

from haitch._attrs import AttributeValue, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

LiElement = NewType("LiElement", Element)
"""A `<li>` element."""


def li(
    *children: Child,
    extra_attrs: Mapping[str, AttributeValue] = {},
    **attrs: Unpack[LiAttrs],
) -> LiElement:
    """Represents an ordered list of items, typically rendered as a blileted list.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/li>
    """
    el = Element("li")(**attrs, **extra_attrs)(*children)
    return LiElement(el)


class LiAttrs(GlobalAttrs):
    """Attributes for the `<li>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/li>
    """

    value: NotRequired[int]
    """Indicates the current ordinal value of the list item as defined by the `<ol>` element.

    The only allowed value for this attribute is a number, even if the list is displayed
    with Roman numerals or letters. List items that follow this one continue numbering
    from the value set. The value attribute has no meaning for unordered lists (`<ul>`)
    or for menus (`<menu>`).

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/li#value>
    """



from __future__ import annotations

from typing import Mapping, NewType

from typing_extensions import Unpack

from haitch._attrs import AttributeValue, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

ArticleElement = NewType("ArticleElement", Element)
"""An `<article>` element."""


def article(
    *children: Child,
    extra_attrs: Mapping[str, AttributeValue] = {},
    **attrs: Unpack[GlobalAttrs],
) -> ArticleElement:
    """Represents self-contained composition in a document, page, application, or site.

    The `<article>` element is intended to be independently distributable or reusable
    (e.g., in syndication). Examples include: a forum post, a magazine or newspaper
    article, or a blog entry, a product card, a user-submitted comment, an interactive
    widget or gadget, or any other independent item of content.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/article>
    """
    el = Element("article")(**attrs, **extra_attrs)(*children)
    return ArticleElement(el)



from __future__ import annotations

from typing import Mapping, NewType

from typing_extensions import Unpack

from haitch._attrs import AttributeValue, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

H1Element = NewType("H1Element", Element)
"""A `<h1>` element."""


def h1(
    *children: Child,
    extra_attrs: Mapping[str, AttributeValue] = {},
    **attrs: Unpack[GlobalAttrs],
) -> H1Element:
    """Represents first level section heading.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/Heading_Elements>
    """
    el = Element("h1")(**attrs, **extra_attrs)(*children)
    return H1Element(el)
